/* 
PTV04 The Doof Machine - Instructables Edition
Peter Polmeartagami - PTV ArtSpace 2023
Create more music machines! Create more music!

Inspired by Gavin Davies' work https://gavd.co.uk/2022/03/arduino-based-drum-machine/
*/

#include <MozziGuts.h>
#include <Sample.h>
#include "sfx/doof01.h"
#include "sfx/doof02.h"
#include "sfx/doof03.h"
#include "sfx/tsh04.h"
#include "sfx/tsh01.h"
#include "sfx/tsh02.h"
#include "sfx/tsh03.h"
#include "sfx/doof04.h"

#include <EventDelay.h>
#include <mozzi_rand.h>

#define CONTROL_RATE 128
#define CELLS_PER_SAMPLE 2048

Sample<CELLS_PER_SAMPLE, AUDIO_RATE> aDoof01(DOOF01_DATA);
Sample<CELLS_PER_SAMPLE, AUDIO_RATE> aDoof02(DOOF02_DATA);
Sample<CELLS_PER_SAMPLE, AUDIO_RATE> aDoof03(DOOF03_DATA);
Sample<CELLS_PER_SAMPLE, AUDIO_RATE> aTsh04(TSH04_DATA);
Sample<CELLS_PER_SAMPLE, AUDIO_RATE> aTsh01(TSH01_DATA);
Sample<CELLS_PER_SAMPLE, AUDIO_RATE> aTsh02(TSH02_DATA);
Sample<CELLS_PER_SAMPLE, AUDIO_RATE> aTsh03(TSH03_DATA);
Sample<CELLS_PER_SAMPLE, AUDIO_RATE> aDoof04(DOOF04_DATA);

Sample<CELLS_PER_SAMPLE, AUDIO_RATE>* sample1 = &aDoof01;
Sample<CELLS_PER_SAMPLE, AUDIO_RATE>* sample2 = &aTsh01;

EventDelay kTriggerDelay;
const byte MAX_STEP_COUNT = 16;
byte cursor1 = 0;
byte cursor2 = 0;

void setup() {
  pinMode(2, INPUT_PULLUP); // ON/OFF switch if you want to add one.
  pinMode(3, INPUT_PULLUP);
  pinMode(4, INPUT_PULLUP);
  pinMode(5, INPUT_PULLUP);
  pinMode(6, INPUT_PULLUP);
  pinMode(7, INPUT_PULLUP);
  pinMode(8, INPUT_PULLUP);
  pinMode(9, OUTPUT);
  pinMode(10, INPUT_PULLUP);
  pinMode(11, INPUT_PULLUP);
  startMozzi();

  aDoof01.setFreq((float)DOOF01_SAMPLERATE / (float)CELLS_PER_SAMPLE);
  aDoof02.setFreq((float)DOOF02_SAMPLERATE / (float)CELLS_PER_SAMPLE);
  aDoof03.setFreq((float)DOOF03_SAMPLERATE / (float)CELLS_PER_SAMPLE);
  aTsh04.setFreq((float)TSH04_SAMPLERATE / (float)CELLS_PER_SAMPLE);
  aTsh01.setFreq((float)TSH01_SAMPLERATE / (float)CELLS_PER_SAMPLE);
  aTsh02.setFreq((float)TSH02_SAMPLERATE / (float)CELLS_PER_SAMPLE);
  aTsh03.setFreq((float)TSH03_SAMPLERATE / (float)CELLS_PER_SAMPLE);
  aDoof04.setFreq((float)DOOF04_SAMPLERATE / (float)CELLS_PER_SAMPLE);
  

  kTriggerDelay.set(10);
}

bool shouldPlay(byte steps, byte beats, byte i) {
  float divisor = ((float)beats / (float)steps);
  bool shouldPlay;
  if (i == 0) {
    shouldPlay = (beats != 0);
  } else {
    byte score = i * divisor;
    byte prevScore = (i - 1) * divisor;
    shouldPlay = (score > prevScore);
  }
  return shouldPlay;
}

void updateControl() {  // ON/OFF switch if you want to add one.
  if (digitalRead(2) == LOW) {
    return;
  }

  if (digitalRead(3) == LOW) {
    sample1 = &aDoof01;
  } else if (digitalRead(4) == LOW) {
    sample1 = &aDoof02;
  } else if (digitalRead(5) == LOW) {
    sample1 = &aDoof03;
  } else if (digitalRead(6) == LOW) {
    sample1 = &aTsh04;
  } else {
    // You can assign different samples or set sample1 to nullptr if you want no sound.
  }

  if (digitalRead(7) == LOW) {
    sample2 = &aTsh01;
  } else if (digitalRead(8) == LOW) {
    sample2 = &aTsh02;
  } else if (digitalRead(10) == LOW) {
    sample2 = &aTsh03;
  } else if (digitalRead(11) == LOW) {
    sample2 = &aDoof04;
  } else {
    // You can assign different samples or set sample2 to nullptr if you want no sound.
  }

  byte steps1 = (byte)map(mozziAnalogRead(4), 0, 1023, 1, MAX_STEP_COUNT);
  byte beats1 = (byte)map(mozziAnalogRead(1), 0, 1023, 0, steps1);
  byte steps2 = (byte)map(mozziAnalogRead(5), 0, 1023, 1, MAX_STEP_COUNT);
  byte beats2 = (byte)map(mozziAnalogRead(2), 0, 1023, 0, steps2);

  if (kTriggerDelay.ready()) {
    if (cursor1 >= steps1) {
      cursor1 = 0;
    }
    if (cursor2 >= steps2) {
      cursor2 = 0;
    }

    if (shouldPlay(steps1, beats1, cursor1)) {
      (*sample1).start();
    }
    if (shouldPlay(steps2, beats2, cursor2)) {
      (*sample2).start();
    }

    cursor1++;
    cursor2++;

    kTriggerDelay.start(map(mozziAnalogRead(3), 0, 1023, 300, 50));
  }
}

int updateAudio() {
  int asig = (int)(((long)(*sample1).next() + (*sample2).next()) * 127);
  if (asig > 127) {
    asig = 127;
  } else if (asig < -128) {
    asig = -128;
  }
  return asig;
}

void loop() {
  audioHook();
}
